@@ -21,15 +21,16 @@ import os
 from pathlib import Path
 import re
 import shlex
-from typing import TYPE_CHECKING, Union
+from typing import TYPE_CHECKING, Any, List, Optional, Tuple, Union
 
 from cylc.flow.hostuserutil import get_host
 from cylc.flow import LOG
-import cylc.flow.flags as flags
+from cylc.flow.exceptions import CylcError
+from cylc.flow.flags import cylc7_back_compat
 from cylc.rose.jinja2_parser import Parser, patch_jinja2_leading_zeros
 from metomi.rose import __version__ as ROSE_VERSION
 from metomi.isodatetime.datetimeoper import DateTimeOperator
-from metomi.rose.config import ConfigDumper, ConfigNodeDiff, ConfigNode
+from metomi.rose.config import ConfigNodeDiff, ConfigNode, ConfigDumper
 from metomi.rose.config_processor import ConfigProcessError
 from metomi.rose.env import env_var_process, UnboundEnvironmentVariableError
 
@@ -42,9 +43,15 @@ SET_BY_CYLC = 'set by Cylc'
 ROSE_ORIG_HOST_INSTALLED_OVERRIDE_STRING = (
     ' ROSE_ORIG_HOST set by cylc install.'
 )
+MESSAGE = 'message'
+ALL_MODES = 'all modes'
 
 
-class MultipleTemplatingEnginesError(Exception):
+class MultipleTemplatingEnginesError(CylcError):
+    ...
+
+
+class InvalidDefineError(CylcError):
     ...
 
 
@@ -190,13 +197,27 @@ def identify_templating_section(config_node):
             "You should not define more than one templating section. "
             f"You defined:\n\t{'; '.join(defined_sections)}"
         )
-    elif 'jinja2:suite.rc' in defined_sections:
-        templating = 'jinja2:suite.rc'
-    elif 'empy:suite.rc' in defined_sections:
-        templating = 'empy:suite.rc'
+    elif defined_sections:
+        return id_templating_section(defined_sections.pop())
     else:
+        return id_templating_section('')
+
+
+def id_templating_section(
+    section: Optional[str] = None,
+    with_brackets: bool = False
+) -> str:
+    """Return a full template section string."""
+    templating = None
+    if section and 'jinja2' in section:
+        templating = 'jinja2:suite.rc'
+    elif section and 'empy' in section:
+        templating = 'empy:suite.rc'
+
+    if not templating:
         templating = 'template variables'
 
+    templating = f'[{templating}]' if with_brackets else templating
     return templating
 
 
@@ -266,7 +287,7 @@ def rose_config_tree_loader(srcdir=None, opts=None):
     # templating section is.)
     if getattr(opts, 'rose_template_vars', None):
         template_section = identify_templating_section(config_tree.node)
-        for template_var in opts.rose_template_vars:
+        for template_var in opts.rose_template_vars or []:
             redefinitions.append(f'[{template_section}]{template_var}')
         # Reload the config
         config_tree = ConfigTreeLoader().load(
@@ -325,6 +346,98 @@ def merge_rose_cylc_suite_install_conf(old, new):
     return old
 
 
+def invalid_defines_check(defines: List) -> None:
+    """Check for defines which do not contain an = and therefore cannot be
+    valid
+
+    Examples:
+
+        # A single invalid define:
+        >>> import pytest
+        >>> with pytest.raises(InvalidDefineError, match=r'\\* foo'):
+        ...    invalid_defines_check(['foo'])
+
+        # Two invalid defines and one valid one:
+        >>> with pytest.raises(
+        ...     InvalidDefineError, match=r'\\* foo.*\\n.* \\* bar'
+        ... ):
+        ...     invalid_defines_check(['foo', 'bar52', 'baz=442'])
+
+        # No invalid defines
+        >>> invalid_defines_check(['foo=12'])
+    """
+    invalid_defines = []
+    for define in defines:
+        if parse_cli_defines(define) is False:
+            invalid_defines.append(define)
+    if invalid_defines:
+        msg = 'Invalid Suite Defines (should contain an =)'
+        for define in invalid_defines:
+            msg += f'\n * {define}'
+        raise InvalidDefineError(msg)
+
+
+def parse_cli_defines(define: str) -> Union[
+    bool, str, Tuple[
+        List[Union[str, Any]],
+        Union[str, Any],
+        Union[str, Any],
+    ]
+]:
+    """Parse a define string.
+
+    Args:
+        define:
+            A string in one of two forms:
+            - `key = "value"`
+            - `[section]key = "value"`
+
+            With optional `!` and `!!` prepended, indicating an ignored state,
+            which should lead to a warning being logged.
+
+    Returns:
+        False: If state is ignored or trigger-ignored, otherwise...
+        (keys, value, state)
+
+    Examples:
+        # Top level key
+        >>> parse_cli_defines('root-dir = "foo"')
+        (['root-dir'], '"foo"', '')
+
+        # Marked as ignored
+        >>> parse_cli_defines('!root-dir = "foo"')
+        False
+
+        # Inside a section
+        >>> parse_cli_defines('[section]orange = "segment"')
+        (['section', 'orange'], '"segment"', '')
+    """
+    match = re.match(
+        (
+            r'^\[(?P<section>.*)\](?P<state>!{0,2})'
+            r'(?P<key>.*)\s*=\s*(?P<value>.*)'
+        ),
+        define
+    )
+    if match:
+        groupdict = match.groupdict()
+        keys = [groupdict['section'].strip(), groupdict['key'].strip()]
+    else:
+        # Doesn't have a section:
+        match = re.match(
+            r'^(?P<state>!{0,2})(?P<key>.*)\s*=\s*(?P<value>.*)', define)
+        if match and not match['state']:
+            groupdict = match.groupdict()
+            keys = [groupdict['key'].strip()]
+        else:
+            # This seems like it ought to be an error,
+            # But behaviour is consistent with Rose 2019
+            # See: https://github.com/cylc/cylc-rose/issues/217
+            return False
+
+    return (keys, match['value'], match['state'])
+
+
 def get_cli_opts_node(opts=None, srcdir=None):
     """Create a ConfigNode representing options set on the command line.
 
@@ -355,38 +468,22 @@ def get_cli_opts_node(opts=None, srcdir=None):
     defines = []
     rose_template_vars = []
     if opts and 'opt_conf_keys' in dir(opts):
-        opt_conf_keys = opts.opt_conf_keys
+        opt_conf_keys = opts.opt_conf_keys or []
     if opts and 'defines' in dir(opts):
-        defines = opts.defines
+        defines = opts.defines or []
     if opts and 'rose_template_vars' in dir(opts):
-        rose_template_vars = opts.rose_template_vars
+        rose_template_vars = opts.rose_template_vars or []
 
     rose_orig_host = get_host()
     defines.append(f'[env]ROSE_ORIG_HOST={rose_orig_host}')
     rose_template_vars.append(f'ROSE_ORIG_HOST={rose_orig_host}')
 
-    # Construct new ouput based on optional Configs:
+    # Construct new config node representing CLI config items:
     newconfig = ConfigNode()
-
-    # For each __define__ determine whether it is an env or template define.
     for define in defines:
-        match = re.match(
-                (
-                    r'^\[(?P<key1>.*)\](?P<state>!{0,2})'
-                    r'(?P<key2>.*)\s*=\s*(?P<value>.*)'
-                ),
-                define
-            ).groupdict()
-        if match['key1'] == '' and match['state'] in ['!', '!!']:
-            LOG.warning(
-                'CLI opts set to ignored or trigger-ignored will be ignored.'
-            )
-        else:
-            newconfig.set(
-                keys=[match['key1'], match['key2']],
-                value=match['value'],
-                state=match['state']
-            )
+        parsed_define = parse_cli_defines(define)
+        if parsed_define:
+            newconfig.set(*parsed_define)
 
     # For each __suite define__ add define.
     if srcdir is not None:
@@ -656,30 +753,50 @@ def deprecation_warnings(config_tree):
         - "root-dir"
         - "jinja2:suite.rc"
         - "empy:suite.rc"
+        - root-dir
 
+    If ALL_MODES is True this deprecation will ignore whether there is a
+    flow.cylc or suite.rc in the workflow directory.
     """
 
     deprecations = {
-        'empy:suite.rc': (
-            "'rose-suite.conf[empy:suite.rc]' is deprecated."
-            " Use [template variables] instead."),
-        'jinja2:suite.rc': (
-            "'rose-suite.conf[jinja2:suite.rc]' is deprecated."
-            " Use [template variables] instead."),
-        'empy:flow.cylc': (
-            "'rose-suite.conf[empy:flow.cylc]' is not used by Cylc."
-            " Use [template variables] instead."),
-        'jinja2:flow.cylc': (
-            "'rose-suite.conf[jinja2:flow.cylc]' is not used by Cylc."
-            " Use [template variables] instead."),
-        'root-dir': (
-            'You have set "rose-suite.conf[root-dir]", '
-            'which is not supported at '
-            'Cylc 8. Use `[install] symlink dirs` in global.cylc '
-            'instead.')
+        'empy:suite.rc': {
+            MESSAGE: (
+                "'rose-suite.conf[empy:suite.rc]' is deprecated."
+                " Use [template variables] instead."),
+            ALL_MODES: False,
+        },
+        'jinja2:suite.rc': {
+            MESSAGE: (
+                "'rose-suite.conf[jinja2:suite.rc]' is deprecated."
+                " Use [template variables] instead."),
+            ALL_MODES: False,
+        },
+        'empy:flow.cylc': {
+            MESSAGE: (
+                "'rose-suite.conf[empy:flow.cylc]' is not used by Cylc."
+                " Use [template variables] instead."),
+            ALL_MODES: False,
+        },
+        'jinja2:flow.cylc': {
+            MESSAGE: (
+                "'rose-suite.conf[jinja2:flow.cylc]' is not used by Cylc."
+                " Use [template variables] instead."),
+            ALL_MODES: False,
+        },
+        'root-dir': {
+            MESSAGE: (
+                'You have set "rose-suite.conf[root-dir]", '
+                'which is not supported at '
+                'Cylc 8. Use `[install] symlink dirs` in global.cylc '
+                'instead.'),
+            ALL_MODES: True,
+        },
     }
-    if not flags.cylc7_back_compat:
-        for string in list(config_tree.node):
-            for deprecation in deprecations.keys():
-                if deprecation in string.lower():
-                    LOG.warning(deprecations[deprecation])
+    for string in list(config_tree.node):
+        for name, info in deprecations.items():
+            if (
+                (info[ALL_MODES] or not cylc7_back_compat)
+                and name in string.lower()
+            ):
+                LOG.warning(info[MESSAGE])
